/***************************************************************/
/* Dustin Graves                                               */
/* CSCI 185 10                                                 */
/* Assignment 3                                                */
/* 10/14/98                                                    */
/***************************************************************/

import java.applet.*;
import java.awt.*;

public class Assignment3 extends Applet implements Runnable
{
  //Data members
  int fNumVerts = 0;                  //Number of vertices total
  int fNumFaces = 0;                  //Number of face in house
  int fWidth;                         //Width of canvas
  int fHeight;                        //Height of canvas
  MyPoly fFaces[] = null;             //Geometric faces
  Point3d fVertices[] = null;         //Vertices
  Xform current = new Xform ();       //Transform to apply to points
  Xform scale = new Xform ();         //Transform to hold scale data
  Xform translate = new Xform ();     //Transform to hold translation data
  Xform rotate = new Xform ();        //Transform to hold rotation data
  Thread fThread = new Thread (this); //Thread to run program
  Image image = null;                 //Off screen buffer for double buffering
  Graphics buffer = null;             //GC for image

  //init
  public void init ()
  {
		String param;
    
    //Read width and height from tag
    param = getParameter("width");
		fWidth = (param == null) ? 300 : Integer.parseInt(param);
    param = getParameter("height");
		fHeight = (param == null) ? 300 : Integer.parseInt(param);

    //Set static values MaxX, MaxY, and MaxZ
    MyGraphicalObject.MaxX = fWidth - 1;
    MyGraphicalObject.MaxY = fHeight - 1;
    MyGraphicalObject.MaxZ = 299;

    //Read the data file
    readFile ("house.dat");

    //Apply the initial scale/translate before paint is called
    applyScale ();

    //Create buffer and start thread
    image = createImage (fWidth, fHeight);
    buffer = image.getGraphics ();
    fThread.start ();
  }  //init

  //Load data file into an array of polygon faces
  public void readFile (String file)
  {
    int i = 0;
    int j = 0;
    int verts = 0;
    int current = 0;
    double x[];
    double y[];
    double z[];
    GWReader reader = new GWReader (file, this);
  
    reader.getString ();

    //Read number of vertices
    fNumVerts = reader.getint ();
    fVertices = new Point3d[fNumVerts];

    //Read number of faces
    fNumFaces = reader.getint ();
    fFaces = new MyPoly[fNumFaces];

    //Read vertices
    for (i = 0; i < fNumVerts; i++)
    {
      fVertices[i] = new Point3d (MyGraphicalObject.xToNormalized (reader.getdouble ()), 
                                  MyGraphicalObject.yToNormalized (reader.getdouble ()), 
                                  MyGraphicalObject.zToNormalized (reader.getdouble ()));
    }

    //Read faces
    for (i = 0; i < fNumFaces; i++)
    {
      //Create arrays and load with points
      verts = reader.getint ();
      x = new double[verts];
      y = new double[verts];
      z = new double[verts];

      for (j = 0; j < verts; j++)
      {
        current = reader.getint ();
        x[j] = fVertices[current-1].fX;
        y[j] = fVertices[current-1].fY;
        z[j] = fVertices[current-1].fZ;
      }

      //Create polygon face
      fFaces[i] = new MyPoly (x, y, z, verts); 
    }
  }  //readFile

  //Method to scale house and center it
  protected void applyScale ()
  {
    int i;

    //Move to origin, scale and move to center
    translate.setTranslate (-8.0/300.0, -8.0/300.0, -42.0/300.0);
    current.setTransform (translate);
    translate.resetTransform ();

    scale.setScale (5.0, 5.0, 5.0);
    current.preCompose (scale);

    translate.setTranslate (0.5, 0.5, 42.0/300.0);
    current.preCompose (translate);

    for (i = 0; i < fNumFaces; i++)
    {
      fFaces[i].apply (current);
    }
  }  //applyScale

  //Method to rotate house by one degree along x and y axes
  protected void applyRotate ()
  {
    int i;

    //Set the current matrix for translate to origin, rotate, and trans back
    translate.setTranslate (-0.5, -0.5, -42.0/300.0);
    current.setTransform (translate);

    rotate.setRotateX (1.0);
    current.preCompose (rotate);
    rotate.setRotateY (1.0);
    current.preCompose (rotate);

    translate.setTranslate (0.5, 0.5, 42.0/300.0);
    current.preCompose (translate);

    for (i = 0; i < fNumFaces; i++)
    {
      fFaces[i].apply (current);
    }
  }  //applyRotate

  public void paint (Graphics g)
  {
    int i;

    buffer.setColor (getBackground ());
    buffer.fillRect (0, 0, fWidth, fHeight);
    buffer.setColor (Color.black);

    for (i = 0; i < fNumFaces; i++)
    {
      fFaces[i].render (buffer);
    }

    //Swap buffers
    g.drawImage (image, 0, 0, this);
  }  //paint

  //Override update to stop g from being cleared everytime repaint is called
  //This (plus the double buffering) will eliminate any blinking.  
  public void update (Graphics g)
  {
    paint (g);
  } //update

  //Method to apply rotation and then update the screen
  public void run ()
  {
    while (fThread != null)
    {
      try 
      {
        fThread.sleep (50);
      }
      catch (Exception e)
      {
        System.out.print ("Interrupt encountered\n");
      }

      applyRotate ();
      repaint ();
    }
  } //run
}
