/***************************************************************/
/* Dustin Graves                                               */
/* CSCI 185 10                                                 */
/* 9/30/98                                                     */
/* 10/14/98                                                    */
/***************************************************************/

import java.awt.*;

//class for geometric transformations
public class Xform
{
  //Instance variables
  protected double fMatrix[][];
  protected boolean fIs3d;
  
  public Xform (boolean is3d)
  {
    fIs3d = is3d;
    fMatrix = new double[4][4];
    setIdentity ();
  }

  public Xform ()
  {
    this (false);
  }

  //Instance methods
  
  //Set the Identity Matrix
  public void setIdentity ()
  {
    int i, j;

    for (i = 0; i < 4; i++)
      for (j = 0; j < 4; j++)
        fMatrix[i][j] = (i == j) ? 1.0 : 0.0;
  }

  //Set ths Xform to Translate 2D
  public void setTranslate (double tx, double ty) 
  {
    setIdentity ();
    fMatrix[0][2] = tx;
    fMatrix[1][2] = ty;

    fIs3d = false;
  }

  //Set ths Xform to Translate 3D
  public void setTranslate (double tx, double ty, double tz) 
  {
    setIdentity ();
    fMatrix[0][3] = tx;
    fMatrix[1][3] = ty;
    fMatrix[2][3] = tz;

    fIs3d = true;
  }

  //Set ths Xform to Scale 2D
  public void setScale (double sx, double sy) 
  {
    setIdentity ();
    fMatrix[0][0] = sx;
    fMatrix[1][1] = sy;

    fIs3d = false;
  }

  //Set ths Xform to Scale 3D
  public void setScale (double sx, double sy, double sz) 
  {
    setIdentity ();
    fMatrix[0][0] = sx;
    fMatrix[1][1] = sy;
    fMatrix[2][2] = sz;

    fIs3d = true;
  }

  //Set ths Xform to Rotate 2D (around Z axis)
  public void setRotate (double theta) 
  {
    //Angle in radians
    double ro = theta * (Math.PI / 180);

    setIdentity ();
    fMatrix[0][0] = Math.cos (ro);
    fMatrix[0][1] = -Math.sin (ro);
    fMatrix[1][0] = Math.sin (ro);
    fMatrix[1][1] = Math.cos (ro);

    fIs3d = false;
  }

  //Set ths Xform to Rotate 3D around X axis
  public void setRotateX (double theta) 
  {
    //Angle in radians
    double ro = theta * (Math.PI / 180);

    setIdentity ();
    fMatrix[1][1] = Math.cos (ro);
    fMatrix[1][2] = -Math.sin (ro);
    fMatrix[2][1] = Math.sin (ro);
    fMatrix[2][2] = Math.cos (ro);

    fIs3d = true;
  }

  //Set ths Xform to Rotate 3D around Y axis
  public void setRotateY (double theta) 
  {
    //Angle in radians
    double ro = theta * (Math.PI / 180);

    setIdentity ();
    fMatrix[0][0] = Math.cos (ro);
    fMatrix[0][2] = Math.sin (ro);
    fMatrix[2][0] = -Math.sin (ro);
    fMatrix[2][2] = Math.cos (ro);

    fIs3d = true;
  }

  //Set ths Xform to Rotate 3D around Z axis
  public void setRotateZ (double theta) 
  {
    //Angle in radians
    double ro = theta * (Math.PI / 180);

    setIdentity ();
    fMatrix[0][0] = Math.cos (ro);
    fMatrix[0][1] = -Math.sin (ro);
    fMatrix[1][0] = Math.sin (ro);
    fMatrix[1][1] = Math.cos (ro);

    fIs3d = true;
  }

  //If this is A and in is B operation is B*A for
  public void preCompose (Xform in)
  {
    //check dimension match
    if (in.fIs3d != fIs3d)
      throw new DimensionError ();

    if (in.fIs3d)
      preCompose3d (in);
    else
      preCompose2d (in);
  }

  //If this is A and in is B operation is B*A for 2D
  public void preCompose2d (Xform in)
  {
    double temp[][] = new double[3][3];
    int r,
        c;

    for (r = 0; r < 3; r++)
    {
      for (c = 0; c < 3; c++)
      {
        temp[r][c] = in.fMatrix[r][0] * fMatrix[0][c] +
                     in.fMatrix[r][1] * fMatrix[1][c] +
                     in.fMatrix[r][2] * fMatrix[2][c];
      }
    }

    for (r = 0; r < 3; r++)
      for (c = 0; c < 3; c++)
        fMatrix[r][c] = temp[r][c];
  }

  //If this is A and in is B operation is B*A for 3D
  public void preCompose3d (Xform in)
  {
    double temp[][] = new double[4][4];
    int r,
        c;

    for (r = 0; r < 4; r++)
    {
      for (c = 0; c < 4; c++)
      {
        temp[r][c] = in.fMatrix[r][0] * fMatrix[0][c] +
                     in.fMatrix[r][1] * fMatrix[1][c] +
                     in.fMatrix[r][2] * fMatrix[2][c] +
                     in.fMatrix[r][3] * fMatrix[3][c];
      }
    }

    for (r = 0; r < 4; r++)
      for (c = 0; c < 4; c++)
        fMatrix[r][c] = temp[r][c];
  }

  //If this is A and in is B operation is A*B for
  public void postCompose (Xform in)
  {
    //check dimension match
    if (in.fIs3d != fIs3d)
      throw new DimensionError ();

    if (in.fIs3d)
      postCompose3d (in);
    else
      postCompose2d (in);
  }

  //If this is A and in is B operation is A*B for 2D
  public void postCompose2d (Xform in)
  {
    double temp[][] = new double[3][3];
    int r,
        c;

    for (r = 0; r < 3; r++)
    {
      for (c = 0; c < 3; c++)
      {
        temp[r][c] = fMatrix[r][0] * in.fMatrix[0][c] +
                     fMatrix[r][1] * in.fMatrix[1][c] +
                     fMatrix[r][2] * in.fMatrix[2][c];
      }
    }

    for (r = 0; r < 3; r++)
      for (c = 0; c < 3; c++)
        fMatrix[r][c] = temp[r][c];
  }

  //If this is A and in is B operation is A*B for 3D
  public void postCompose3d (Xform in)
  {
    double temp[][] = new double[4][4];
    int r,
        c;

    for (r = 0; r < 4; r++)
    {
      for (c = 0; c < 4; c++)
      {
        temp[r][c] = fMatrix[r][0] * in.fMatrix[0][c] +
                     fMatrix[r][1] * in.fMatrix[1][c] +
                     fMatrix[r][2] * in.fMatrix[2][c] +
                     fMatrix[r][3] * in.fMatrix[3][c];
      }
    }

    for (r = 0; r < 4; r++)
      for (c = 0; c < 4; c++)
        fMatrix[r][c] = temp[r][c];
  }

  //Apply this Xform to a point
  public Point2d apply (Point2d in)
  {
    double tempx;

    //Check dimension
    if (fIs3d)
      throw new DimensionError ();

    tempx = (fMatrix[0][0] * in.fX) +
            (fMatrix[0][1] * in.fY) +
            (fMatrix[0][2] * in.fH);

    in.fY = (fMatrix[1][0] * in.fX) +
            (fMatrix[1][1] * in.fY) +
            (fMatrix[1][2] * in.fH);

    in.fX = tempx;

    return in;
  }

  //Apply this Xform to a point
  public Point3d apply (Point3d in)
  {
    double tempx,
           tempy;

    //Check dimension
    if (!fIs3d)
      throw new DimensionError ();

    tempx = (fMatrix[0][0] * in.fX) +
            (fMatrix[0][1] * in.fY) +
            (fMatrix[0][2] * in.fZ) +
            (fMatrix[0][3] * in.fH);

    tempy = (fMatrix[1][0] * in.fX) +
            (fMatrix[1][1] * in.fY) +
            (fMatrix[1][2] * in.fZ) +
            (fMatrix[1][3] * in.fH);

    in.fZ = (fMatrix[2][0] * in.fX) +
            (fMatrix[2][1] * in.fY) +
            (fMatrix[2][2] * in.fZ) +
            (fMatrix[2][3] * in.fH);

    in.fX = tempx;
    in.fY = tempy;

    return in;
  }

  public void setTransform (Xform in)
  {
    int c,
        r;

    fIs3d = in.fIs3d;

    for (r = 0; r < 4; r++)
      for (c = 0; c < 4; c++)
        fMatrix[r][c] = in.fMatrix[r][c];
  }

  public void resetTransform ()
  {
    setIdentity ();
  }

} //Xform



